@@ -92,22 +92,22 @@ class TelegramBot(MultiBot[TelegramClient]):
         return bot_message
 
     @return_if_first_empty(exclude_self_types='TelegramBot', globals_=globals())
-    async def _create_chat_from_telegram_chat(self, telegram_chat: constants.TELEGRAM_CHAT) -> Chat | None:
-        chat_name = self._get_name_from_entity(telegram_chat)
-        if isinstance(telegram_chat, constants.TELEGRAM_USER):
+    async def _create_chat_from_telegram_chat(self, original_chat: constants.TELEGRAM_CHAT) -> Chat | None:
+        chat_name = self._get_name_from_entity(original_chat)
+        if isinstance(original_chat, constants.TELEGRAM_USER):
             group_id = None
             group_name = None
         else:
-            group_id = telegram_chat.id
+            group_id = original_chat.id
             group_name = chat_name
 
         return Chat(
             platform=self.platform.value,
-            id=telegram_chat.id,
+            id=original_chat.id,
             name=chat_name,
             group_id=group_id,
             group_name=group_name,
-            original_object=telegram_chat
+            original_object=original_chat
         )
 
     @return_if_first_empty(exclude_self_types='TelegramBot', globals_=globals())
@@ -123,6 +123,7 @@ class TelegramBot(MultiBot[TelegramClient]):
             id=original_user.id,
             name=self._get_name_from_entity(original_user).strip(' @'),
             is_admin=is_admin,
+            is_bot=original_user.bot,
             original_object=original_user
         )
 
@@ -319,11 +320,6 @@ class TelegramBot(MultiBot[TelegramClient]):
 
         return await self._create_chat_from_telegram_chat(await self.client.get_entity(chat_id))
 
-    @return_if_first_empty(exclude_self_types='TelegramBot', globals_=globals())
-    async def get_group_users(self, group_: int | str | Chat | Message) -> list[User]:
-        chat = await self.get_chat(group_)
-        return [await self._create_user_from_telegram_user(participant, chat.id) for participant in await self.client.get_participants(chat.original_object)]
-
     async def get_me(self, group_: int | str | Chat = None):
         return await self._create_user_from_telegram_user(await self.client.get_me(), group_)
 
@@ -334,6 +330,11 @@ class TelegramBot(MultiBot[TelegramClient]):
         with flanautils.suppress_stderr():
             return await self._create_user_from_telegram_user(await self.client.get_entity(user), group_id)
 
+    @return_if_first_empty(exclude_self_types='TelegramBot', globals_=globals())
+    async def get_users(self, group_: int | str | Chat | Message) -> list[User]:
+        chat = await self.get_chat(group_)
+        return [await self._create_user_from_telegram_user(participant, chat.id) for participant in await self.client.get_participants(chat.original_object)]
+
     @parse_arguments
     async def send(
         self,
@@ -362,8 +363,6 @@ class TelegramBot(MultiBot[TelegramClient]):
             return telegram_buttons
 
         file = await self._prepare_media_to_send(media)
-        if not text and not file:
-            return
 
         kwargs = {
             'file': file,
