@@ -1,5 +1,7 @@
 from __future__ import annotations  # todo0 remove in 3.11
 
+__all__ = ['TwitchBot']
+
 import asyncio
 import datetime
 import re
@@ -9,9 +11,10 @@ from typing import Iterable, Iterator
 import flanautils
 import pymongo
 import twitchio
+import twitchio.ext.commands
 from flanautils import Media, OrderedSet, return_if_first_empty
 
-import constants
+from multibot import constants
 from multibot.bots.multi_bot import MultiBot, parse_arguments
 from multibot.models import Chat, Message, Platform, User
 
@@ -22,7 +25,7 @@ from multibot.models import Chat, Message, Platform, User
 class TwitchBot(MultiBot[twitchio.Client]):
     def __init__(self, bot_token: str, initial_channels: Iterable[str] = None, owner_name: str = None):
         super().__init__(bot_token=bot_token,
-                         bot_client=twitchio.Client(token=bot_token, initial_channels=initial_channels))
+                         bot_client=twitchio.ext.commands.Bot(token=bot_token, prefix='/', initial_channels=initial_channels))
         self.owner_name = owner_name
 
     # ----------------------------------------------------------- #
@@ -36,11 +39,11 @@ class TwitchBot(MultiBot[twitchio.Client]):
         self.client._events['event_message'].append(self._on_new_message_raw)
 
     @return_if_first_empty(exclude_self_types='TwitchBot', globals_=globals())
-    async def _create_chat_from_twitch_chat(self, twitch_chat: constants.TWITCH_CHAT) -> Chat | None:
-        channel_name = twitch_chat.name
+    async def _create_chat_from_twitch_chat(self, original_chat: constants.TWITCH_CHAT) -> Chat | None:
+        channel_name = original_chat.name
         try:
-            channel_id = int(flanautils.find(twitch_chat.chatters, condition=lambda user: user.name == channel_name).id)
-        except AttributeError:
+            channel_id = int(flanautils.find(original_chat.chatters, condition=lambda user: user.name == channel_name).id)
+        except (AttributeError, TypeError):
             channel_id = int(next(iter(await self.client.fetch_users([channel_name])), None).id)
 
         return Chat(
@@ -49,20 +52,20 @@ class TwitchBot(MultiBot[twitchio.Client]):
             name=channel_name,
             group_id=channel_id,
             group_name=channel_name,
-            original_object=twitch_chat
+            original_object=original_chat
         )
 
     @return_if_first_empty(exclude_self_types='TwitchBot', globals_=globals())
-    async def _create_user_from_twitch_user(self, twitch_user: constants.TWITCH_USER, is_admin: bool = None) -> User | None:
-        if (id := twitch_user.id) is None:
-            id = next(iter(await self.client.fetch_users([twitch_user.name])), None).id
+    async def _create_user_from_twitch_user(self, original_user: constants.TWITCH_USER, is_admin: bool = None) -> User | None:
+        if (id := original_user.id) is None:
+            id = next(iter(await self.client.fetch_users([original_user.name])), None).id
 
         return User(
             platform=self.platform.value,
             id=int(id),
-            name=twitch_user.name,
-            is_admin=getattr(twitch_user, 'is_mod', None) if is_admin is None else is_admin,
-            original_object=twitch_user
+            name=original_user.display_name,
+            is_admin=getattr(original_user, 'is_mod', None) if is_admin is None else is_admin,
+            original_object=original_user
         )
 
     @return_if_first_empty(exclude_self_types='TwitchBot', globals_=globals())
@@ -77,15 +80,16 @@ class TwitchBot(MultiBot[twitchio.Client]):
 
     @return_if_first_empty(exclude_self_types='TwitchBot', globals_=globals())
     async def _get_mentions(self, original_message: constants.TWITCH_MESSAGE) -> list[User]:
-        mentions = OrderedSet([user for mention in re.findall(r'@[\d\w]+', original_message.content) if (user := await self.get_user(mention[1:], original_message.channel.name))])
-
         text = await self._get_text(original_message)
         chat = await self._get_chat(original_message)
+        mentions = OrderedSet([user for mention in re.findall(r'@[\d\w]+', text) if (user := await self.get_user(mention[1:], chat))])
 
-        words = text.lower().split()
-        for chatter in chat.original_object.chatters:
-            if chatter.name.lower() in words:
-                mentions.add(await self._create_user_from_twitch_user(chatter))
+        if chat.original_object.chatters:
+            words = text.lower().split()
+
+            for chatter in chat.original_object.chatters:
+                if chatter.name.lower() in words:
+                    mentions.add(await self._create_user_from_twitch_user(chatter))
 
         return list(mentions)
 
@@ -106,7 +110,7 @@ class TwitchBot(MultiBot[twitchio.Client]):
 
     @return_if_first_empty(exclude_self_types='TwitchBot', globals_=globals())
     async def _get_text(self, original_message: constants.TWITCH_MESSAGE) -> str:
-        return re.sub(r'@[\d\w]+', '', original_message.content).strip()
+        return original_message.content
 
     # ---------------------------------------------- #
     #                    HANDLERS                    #
@@ -171,7 +175,7 @@ class TwitchBot(MultiBot[twitchio.Client]):
         message_to_delete.save()
 
     @return_if_first_empty(exclude_self_types='TwitchBot', globals_=globals())
-    async def get_chat(self, chat: int | str | Chat | Message = None) -> Chat | None:
+    async def get_chat(self, chat: int | str | User | Chat | Message = None) -> Chat | None:
         match chat:
             case int(group_id):
                 group_name = self.get_group_name(group_id)
@@ -180,6 +184,8 @@ class TwitchBot(MultiBot[twitchio.Client]):
                     return await self._create_chat_from_twitch_chat(await self.client.fetch_channel(group_id))
             case str(group_name):
                 pass
+            case User() as user:
+                group_name = user.name.lower()
             case Chat():
                 return chat
             case Message() as message:
@@ -190,7 +196,7 @@ class TwitchBot(MultiBot[twitchio.Client]):
         return await self._create_chat_from_twitch_chat(self.client.get_channel(group_name) or await self.client.fetch_channel(group_name))
 
     @return_if_first_empty(exclude_self_types='TwitchBot', globals_=globals())
-    async def get_group_users(self, group_: int | str | Chat | Message) -> list[User]:
+    async def get_users(self, group_: int | str | Chat | Message) -> list[User]:
         chat = await self.get_chat(group_)
         return list(OrderedSet([await self._create_user_from_twitch_user(chatter) for chatter in chat.original_object.chatters]))
 
@@ -206,36 +212,51 @@ class TwitchBot(MultiBot[twitchio.Client]):
             case _ as user_id_or_name:
                 pass
 
-        twitch_user: twitchio.User | twitchio.Chatter
-        if not (twitch_user := next(iter(await self.client.fetch_users([user_id_or_name])), None)):
+        original_user: twitchio.User | twitchio.Chatter
+        if not (original_user := next(iter(await self.client.fetch_users([user_id_or_name])), None)):
             return
 
         if group_name:
-            if twitch_user.name == group_name:
+            if original_user.name == group_name:
                 is_admin = True
             else:
                 chat = await self.get_chat(group_)
-                twitch_user = chat.original_object.get_chatter(twitch_user.name)
+                original_user = chat.original_object.get_chatter(original_user.name)
                 is_admin = None
         else:
             is_admin = None
-        return await self._create_user_from_twitch_user(twitch_user, is_admin=is_admin)
+        return await self._create_user_from_twitch_user(original_user, is_admin=is_admin)
 
     async def join(self, chat_name: str | Iterable[str]):
         await self.client.join_channels((chat_name,) if isinstance(chat_name, str) else chat_name)
 
+    async def leave(self, chat_name: str | Iterable[str]):
+        await self.client.part_channels((chat_name,) if isinstance(chat_name, str) else chat_name)
+
     @parse_arguments
     async def send(
         self,
         text='',
         media: Media = None,
         buttons: list[str | list[str]] | None = None,
+        chat: int | str | User | Chat | Message | None = None,
         message: Message = None,
+        *,
+        reply_to: str | Message = None,
         silent: bool = False,
         send_as_file: bool = None,
         edit=False,
     ):
-        await message.chat.original_object.send(text)
+        match reply_to:
+            case str(message_id):
+                # noinspection PyProtectedMember
+                await message.chat.original_object._ws.reply(message_id, f"PRIVMSG #{message.author.name.lower()} :{text}\r\n")
+            case Message() as message_to_reply:
+                # noinspection PyUnresolvedReferences
+                context = await self.client.get_context(message_to_reply.original_object)
+                await context.reply(text)
+            case _:
+                await chat.original_object.send(text)
 
     def start(self):
         async def start_():
